from django.conf import settings
from django.shortcuts import render
from django.template.loader import render_to_string
from django.core.mail import EmailMessage
from django.forms.utils import ErrorList
from django.utils.safestring import mark_safe

import logging
import functools

from django.contrib.auth.models import User
from pages.models import PageWeb
from gestion.inscriptions.models import (
    enregistrer_photos,
    preparer_inscription,
    redimensionner_photo,
    MODELES_CHAMP,
    DonneeFormulaire,
    LdapCriError,
    EtatInscr,
)
from gestion.inscriptions.creation_comptes import prochain_uid_libre, creer_compte
from gestion.inscriptions.forms import InscriptionForm
from .logins import vers_alphabet, genere_login
from .forms import InscriptionLimiteeForm
from lib.tools import erreur
from portailuser.models import Ecole
from ldap_inp.utf_tools import to_alphabetic

from PIL import Image

from typing import Optional, Union

_logger = logging.getLogger("inscriptions_directes")


def limite_inscriptions_ecoles(fun):
    @functools.wraps(fun)
    def decorator(request):
        if request.ecole.autoinscription is False:
            return erreur(
                request,
                "Désolé, le service d'inscriptions en ligne n'a pas été activé par votre BDE, merci de les contacter directement pour vous inscrire.",
                "Accès refusé",
            )
        return fun(request)

    return decorator


def valider_formulaire(form: Union[InscriptionLimiteeForm, InscriptionForm]) -> Optional[str]:
    """ Retourne le nouvel identifiant, ou None en cas d'erreur. """

    if len(vers_alphabet(form.cleaned_data["nom"])) == 0:
        form.errors["nom"] = ErrorList(["Ce nom ne contient pas de caractères alphabétiques."])
        return None

    if len(vers_alphabet(form.cleaned_data["prenom"])) == 0:
        form.errors["prenom"] = ErrorList(["Ce prénom ne contient pas de caractères alphabétiques."])
        return None

    if isinstance(form, InscriptionLimiteeForm) or "login" not in form.cleaned_data.keys():
        login = genere_login(form.cleaned_data["nom"], form.cleaned_data["prenom"])
    else:
        login = form.cleaned_data["login"]

    if login is None:
        form.errors["nom"] = ErrorList(["Impossible de générer un login unique !"])
        return None

    email = form.cleaned_data["email"].strip()

    for domaine_ecole in settings.DOMAINES_INSTITUTIONNELS:
        if email.endswith(domaine_ecole):
            form.errors["email"] = ErrorList(
                ["L'adresse email institutionnelle ne peut pas être utilisée comme adresse email annexe"]
            )
            return None

    return login


def afficher_page_nouvelle_inscription(request, form: InscriptionLimiteeForm = None):
    try:
        page = PageWeb.objects.get(ecole=request.ecole, adresse="ae/inscriptions")
    except PageWeb.DoesNotExist:
        page = None

    if form is None:
        form = InscriptionLimiteeForm(ecole=request.ecole)

    c = {
        "page": page,
        "peut_editer": request.user.is_authenticated and request.user.check_permission(request, "gerer_pages"),
        "form": form,
    }

    return render(request, "inscriptions/nouvelle.html", c)


@limite_inscriptions_ecoles
def nouvelle(request):
    if request.method != "POST":
        return afficher_page_nouvelle_inscription(request, InscriptionLimiteeForm(ecole=request.ecole))

    form = InscriptionLimiteeForm(request.ecole, request.POST, request.FILES)
    if not form.is_valid():
        return afficher_page_nouvelle_inscription(request, form)

    login = valider_formulaire(form)
    if login is None:
        return afficher_page_nouvelle_inscription(request, form)

    inscr = form.save(commit=False)

    inscr.ecole = request.ecole
    inscr.login = login
    inscr.mdp = User.objects.make_random_password(length=20)
    inscr.etat = 0

    inscr_limitee = inscr

    inscr = inscr.save()

    for type_champ in MODELES_CHAMP.values():
        for champ in type_champ.objects.filter(ecole=request.ecole, inscription_directe=True):
            try:
                donnee = DonneeFormulaire.objects.get(inscription=inscr, ident_champ=champ.ident)
            except DonneeFormulaire.DoesNotExist:
                donnee = DonneeFormulaire(inscription=inscr, ident_champ=champ.ident)
            donnee.valeur = None if form.cleaned_data[champ.ident] is None else str(form.cleaned_data[champ.ident])
            donnee.save()

    # TODO: retravailler le code.
    # Code fragile, puisque basé sur des détails d'implantation de Pillow
    if inscr_limitee.photo._file is not None:
        photo = Image.open(inscr_limitee.photo._file, "r")
        donnees = redimensionner_photo(photo)
        enregistrer_photos((inscr_limitee.login, donnees, []), inscr.id)

    try:
        (code, message) = preparer_inscription(inscr, tout_revalider=True, avec_cri=request.ecole.ldap_cri)
        inscr.code = code
        inscr.message = mark_safe(message) if message else message
        if code != EtatInscr.ERREUR:
            inscr.etat = 2
        inscr.save()
        # erreur -> validation manuelle
        if code == EtatInscr.OK:
            uid = prochain_uid_libre()
            if request.ecole == Ecole.enm:
                prenom = to_alphabetic(inscr.prenom)
                nom = to_alphabetic(inscr.nom)
                inscr.ident_cri = f"{prenom}.{nom}"
                inscr.mail_cri = f"{prenom}.{nom}@meteo.fr"
            creer_compte(inscr, uid, "ou=people," + request.ecole.dn)
            inscr.etat = 3
            inscr.save()

            return render(request, "inscriptions/success.html")
    except LdapCriError:
        print("Erreur de requète sur le LDAP du CRI")
        # échec de requète LDAP -> validation manuelle
        pass

    # Envoi d'un mail pour que l'utilisateur ne spamme pas les demandes de création de compte
    message = render_to_string("inscriptions/mail_confirmation_inscription.txt", {"inscr": inscr})
    EmailMessage(
        "Inscription à INP-net",
        message,
        "INP-net <noreply@bde.inp-toulouse.fr>",
        [f"{inscr.prenom} {inscr.nom} <{inscr.email}>"],
        headers={"Reply-To": "INP-net <inp-net@bde.inp-toulouse.fr>"},
    ).send(fail_silently=False)
    _logger.info(f"Demande de création du compte '{inscr.nom} {inscr.prenom}'")

    return render(request, "inscriptions/success.html")
